/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include <qlayout.h>
#include <qinputdialog.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qt.h>

#include "configdialog.h"
#include "paths.h"

CheckBoxItem::CheckBoxItem(Settings* settings, unsigned int index, QListView* parent)
	: QCheckListItem(parent, settings->sites()[index].title, QCheckListItem::CheckBox)
{
	this->setOn(settings->sites()[index].enabled);
	this->settings = settings;
	this->_index = index;
}

int
CheckBoxItem::index()
{
	return _index;
}

void
CheckBoxItem::setIndex(int index)
{
	_index = index;
}

Site
CheckBoxItem::site()
{
	return settings->sites()[_index];
}

bool
CheckBoxItem::acceptDrop(const QMimeSource* mime) const
{
	return QTextDrag::canDecode(mime);
}

void
CheckBoxItem::dropped(QDropEvent* e)
{
	QString url;
	QTextDrag::decode(e, url);
	for (unsigned int i = 0; i < settings->sites().count(); i++) {
		if (settings->sites()[i].url == url) {
			settings->indexSite(settings->sites()[i], _index);
			return;
		}
	}
}

int
CheckBoxItem::compare(QListViewItem* item, int, bool) const
{
	CheckBoxItem* i = (CheckBoxItem*)item;
	if (_index < i->_index)
		return -1;
	if (_index > i->_index)
		return 1;
	return 0;
}

void
CheckBoxItem::promote()
{
	if (_index == 0)
		return;
	_index--;
	settings->indexSite(settings->sites()[_index+1], _index);
}

void
CheckBoxItem::demote()
{
	if (_index == settings->sites().count() - 1)
		return;
	_index++;
	settings->indexSite(settings->sites()[_index-1], _index);
}

ConfigDialog::ConfigDialog(Settings* settings, Cache* cache, QWidget* parent, const char* name)
	: QTabDialog(parent, name, false, Qt::WStyle_StaysOnTop)
{
	this->settings = settings;
	
	setCaption(tr("Properties"));
	setOkButton();
	
	this->adder = new Adder(cache, true, this);
	connect(adder, SIGNAL(validSite(Site)), settings, SLOT(appendSite(Site)));
	connect(adder, SIGNAL(invalidUrl(QString)), this, SLOT(urlFailed(QString)));
	
	QWidget* rsstab = new QWidget(this);
	QVBoxLayout* rsstablayout = new QVBoxLayout(rsstab, 8);
	QHBoxLayout* rsslistlayout = new QHBoxLayout(rsstablayout);
	rsslist = new SiteListView(rsstab);
	rsslist->addColumn("whatever");
	rsslist->header()->hide();
	rsslist->viewport()->setAcceptDrops(true);
	connect(rsslist, SIGNAL(clicked(QListViewItem*)), this, SLOT(rssEnable(QListViewItem*)));
	connect(rsslist, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(rssEnable(QListViewItem*)));
	connect(rsslist, SIGNAL(spacePressed(QListViewItem*)), this, SLOT(rssEnable(QListViewItem*)));
	rsslistlayout->addWidget(rsslist);
	
	QVBoxLayout* rssorderlayout = new QVBoxLayout(rsslistlayout);
	rssupbutton = new QPushButton(trUtf8("\xe2\x86\x91"), rsstab);
	rssupbutton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding, 0, 0, false));
	rssupbutton->setMaximumWidth(24);
	connect(rssupbutton, SIGNAL(clicked()), this, SLOT(promoteRss()));
	rssorderlayout->addWidget(rssupbutton);
	rssdownbutton = new QPushButton(trUtf8("\xe2\x86\x93"), rsstab);
	rssdownbutton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding, 0, 0, false));
	rssdownbutton->setMaximumWidth(24);
	connect(rssdownbutton, SIGNAL(clicked()), this, SLOT(demoteRss()));
	rssorderlayout->addWidget(rssdownbutton);
	
	QHBoxLayout* rsscontrollayout = new QHBoxLayout(rsstablayout);
	QVBoxLayout* rsslistbuttonslayout = new QVBoxLayout(rsscontrollayout);
	QPushButton* rssaddbutton = new QPushButton(tr("Add"), rsstab);
	rssaddbutton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	rsslistbuttonslayout->addWidget(rssaddbutton);
	connect(rssaddbutton, SIGNAL(clicked()), this, SLOT(addRss()));
	rssremovebutton = new QPushButton(tr("Remove"), rsstab);
	rssremovebutton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(rssremovebutton, SIGNAL(clicked()), this, SLOT(removeRss()));
	rsslistbuttonslayout->addWidget(rssremovebutton);
	connect(settings, SIGNAL(sitesChange(SiteList)),
	        this, SLOT(updateRssList(SiteList)));
	rsslistbuttonslayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Minimum));
	rsscontrollayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
	
	QVBoxLayout* rssreloadlayout = new QVBoxLayout(rsscontrollayout);
	QHBoxLayout* rssreloadspinlayout = new QHBoxLayout(rssreloadlayout);
	QLabel* rssreloadlabel = new QLabel(tr("Reload Interval"), rsstab);
	rssreloadspinlayout->addWidget(rssreloadlabel);
	QSpinBox* rssreloadspin = new QSpinBox(30, 120, 5, rsstab);
	rssreloadspin->setSuffix(tr("mins"));
	rssreloadspinlayout->addWidget(rssreloadspin);
	connect(settings, SIGNAL(cacheTimeoutChange(int)), rssreloadspin, SLOT(setValue(int)));
	connect(rssreloadspin, SIGNAL(valueChanged(int)), cache, SLOT(setTimeout(int)));
	rssreloadslider = new QSlider(30, 120, 5, settings->cacheTimeout(), Qt::Horizontal, rsstab);
	rssreloadlayout->addWidget(rssreloadslider);
	connect(rssreloadspin, SIGNAL(valueChanged(int)), rssreloadslider, SLOT(setValue(int)));
	connect(rssreloadslider, SIGNAL(valueChanged(int)), rssreloadspin, SLOT(setValue(int)));
	rssreloadlayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum));
	rsscontrollayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
	
	QVBoxLayout* monitoringlayout = new QVBoxLayout(rsscontrollayout);
	QCheckBox* monitoring = new QCheckBox(tr("Monitor Clipboard"), rsstab);
	monitoringlayout->addWidget(monitoring);
	connect(monitoring, SIGNAL(toggled(bool)), settings, SLOT(enableMonitoring(bool)));
	connect(settings, SIGNAL(monitoringChange(bool)),
	        monitoring, SLOT(setChecked(bool)));
	monitoringlayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Minimum));
	
	this->addTab(rsstab, tr("RSS"));
	
	QWidget* appearancetab = new QWidget(this);
	QVBoxLayout* appearancelayout = new QVBoxLayout(appearancetab, 8);
	QHBoxLayout* skinlayout = new QHBoxLayout(appearancelayout);
	QLabel* skinlabel = new QLabel(tr("Skin"), appearancetab);
	skinlayout->addWidget(skinlabel);
	skincombo = new QComboBox(appearancetab);
	skincombo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	skincombo->insertStringList(Skin::available());
	skincombo->setCurrentText(settings->skin());
	connect(skincombo, SIGNAL(highlighted(const QString&)), settings, SLOT(setSkin(const QString&)));
	connect(skincombo, SIGNAL(activated(const QString&)), settings, SLOT(setSkin(const QString&)));
	connect(settings, SIGNAL(skinChange(const QString&)), this, SLOT(setSkin(const QString&)));
	skinlayout->addWidget(skincombo);
	
	QGroupBox* tickergroup = new QGroupBox(tr("Ticker"), appearancetab);
	tickergroup->setColumnLayout(0, Qt::Vertical);
	QHBoxLayout* tickerlayout = new QHBoxLayout(tickergroup->layout(), 8);
	QVBoxLayout* fontlayout = new QVBoxLayout(tickerlayout);
	QLabel* facelabel = new QLabel(tr("Font Family"), tickergroup);
	fontlayout->addWidget(facelabel);
	facecombo = new QComboBox(tickergroup);
	facecombo->insertStringList(fdb.families());
	facecombo->setCurrentText(settings->fontFace());
	connect(facecombo, SIGNAL(highlighted(const QString&)), settings, SLOT(setFontFace(const QString&)));
	connect(facecombo, SIGNAL(activated(const QString&)), settings, SLOT(setFontFace(const QString&)));
	connect(settings, SIGNAL(fontFaceChange(const QString&)), this, SLOT(faceChange(const QString&)));
	fontlayout->addWidget(facecombo);
	QLabel* sizelabel = new QLabel(tr("Font Size"), tickergroup);
	fontlayout->addWidget(sizelabel);
	sizecombo = new QComboBox(tickergroup);
	for (unsigned int i = 0; i < fdb.standardSizes().count(); i++) {
		sizecombo->insertItem(QString::number(fdb.standardSizes()[i]));
	}
	sizeChange(settings->fontSize());
	connect(sizecombo, SIGNAL(highlighted(const QString&)), this, SLOT(changeSize(const QString&)));
	connect(sizecombo, SIGNAL(activated(const QString&)), this, SLOT(changeSize(const QString&)));
	connect(settings, SIGNAL(fontSizeChange(int)), this, SLOT(sizeChange(int)));
	fontlayout->addWidget(sizecombo);
	QHBoxLayout* autofontlayout = new QHBoxLayout(fontlayout);
	autofontlayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
	QCheckBox* autofontcheck = new QCheckBox(tr("Default"), tickergroup);
	connect(autofontcheck, SIGNAL(toggled(bool)), facecombo, SLOT(setDisabled(bool)));
	connect(autofontcheck, SIGNAL(toggled(bool)), sizecombo, SLOT(setDisabled(bool)));
	connect(autofontcheck, SIGNAL(toggled(bool)), settings, SLOT(setAutoFont(bool)));
	connect(settings, SIGNAL(autoFontChange(bool)), autofontcheck, SLOT(setChecked(bool)));
	autofontlayout->addWidget(autofontcheck);
	
	QVBoxLayout* colorlayout = new QVBoxLayout(tickerlayout);
	QLabel* fglabel = new QLabel(tr("Text Color"), tickergroup);
	colorlayout->addWidget(fglabel);
	fgbutton = new QPushButton(tickergroup);
	connect(fgbutton, SIGNAL(clicked()), this, SLOT(changeFg()));
	connect(settings, SIGNAL(fgChange(QColor)), this, SLOT(fgChange(QColor)));
	colorlayout->addWidget(fgbutton);
	QLabel* bglabel = new QLabel(tr("Background Color"), tickergroup);
	colorlayout->addWidget(bglabel);
	bgbutton = new QPushButton(tickergroup);
	connect(bgbutton, SIGNAL(clicked()), this, SLOT(changeBg()));
	connect(settings, SIGNAL(bgChange(QColor)), this, SLOT(bgChange(QColor)));
	colorlayout->addWidget(bgbutton);
	QHBoxLayout* autocolorlayout = new QHBoxLayout(colorlayout);
	autocolorlayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
	QCheckBox* autocolorcheck = new QCheckBox(tr("Automatic"), tickergroup);
	connect(autocolorcheck, SIGNAL(toggled(bool)), fgbutton, SLOT(setDisabled(bool)));
	connect(autocolorcheck, SIGNAL(toggled(bool)), bgbutton, SLOT(setDisabled(bool)));
	connect(autocolorcheck, SIGNAL(toggled(bool)), settings, SLOT(setAutoColors(bool)));
	connect(settings, SIGNAL(autoColorsChange(bool)), autocolorcheck, SLOT(setChecked(bool)));
	autocolorlayout->addWidget(autocolorcheck);
	appearancelayout->addWidget(tickergroup);
	
	QGroupBox* interactivitygroup = new QGroupBox(tr("Interactivity"), appearancetab);
	interactivitygroup->setColumnLayout(0, Qt::Vertical);
	QVBoxLayout* interactivitylayout = new QVBoxLayout(interactivitygroup->layout(), 8);
	QHBoxLayout* intercontrolslayout = new QHBoxLayout(interactivitylayout);
	QVBoxLayout* headlineslayout = new QVBoxLayout(intercontrolslayout);
	QHBoxLayout* timeoutlayout = new QHBoxLayout(headlineslayout);
	QLabel* timeoutlabel = new QLabel(tr("Headline Timeout"), interactivitygroup);
	timeoutlayout->addWidget(timeoutlabel);
	QSpinBox* timeoutspin = new QSpinBox(5, 60, 5, interactivitygroup);
	timeoutspin->setSuffix(tr("secs"));
	connect(settings, SIGNAL(headlineTimeoutChange(int)), timeoutspin, SLOT(setValue(int)));
	connect(timeoutspin, SIGNAL(valueChanged(int)), settings, SLOT(setHeadlineTimeout(int)));
	timeoutlayout->addWidget(timeoutspin);
	QSlider* timeoutslider = new QSlider(5, 60, 5, settings->headlineTimeout(), Qt::Horizontal, interactivitygroup);
	connect(timeoutspin, SIGNAL(valueChanged(int)), timeoutslider, SLOT(setValue(int)));
	connect(timeoutslider, SIGNAL(valueChanged(int)), timeoutspin, SLOT(setValue(int)));
	headlineslayout->addWidget(timeoutslider);
	QCheckBox* underlinecheck = new QCheckBox(tr("Underline links"), interactivitygroup);
	connect(underlinecheck, SIGNAL(toggled(bool)), settings, SLOT(enableUnderlining(bool)));
	connect(settings, SIGNAL(underliningChange(bool)), underlinecheck, SLOT(setChecked(bool)));
	headlineslayout->addWidget(underlinecheck);
	QCheckBox* tooltipscheck = new QCheckBox(tr("Display tooltips"), interactivitygroup);
	connect(tooltipscheck, SIGNAL(toggled(bool)), settings, SLOT(enableTooltips(bool)));
	connect(settings, SIGNAL(tooltipsChange(bool)), tooltipscheck, SLOT(setChecked(bool)));
	headlineslayout->addWidget(tooltipscheck);
	
	QVBoxLayout* scrollinglayout = new QVBoxLayout(intercontrolslayout);
	QHBoxLayout* scrollingspinlayout = new QHBoxLayout(scrollinglayout);
	QLabel* scrollinglabel = new QLabel(tr("Scroll Speed"), interactivitygroup);
	scrollingspinlayout->addWidget(scrollinglabel);
	QSpinBox* scrollingspin = new QSpinBox(200, 2000, 100, interactivitygroup);
	scrollingspin->setSuffix(tr("ms"));
	connect(settings, SIGNAL(scrollTimeoutChange(int)), scrollingspin, SLOT(setValue(int)));
	connect(scrollingspin, SIGNAL(valueChanged(int)), settings, SLOT(setScrollTimeout(int)));
	scrollingspinlayout->addWidget(scrollingspin);
	QSlider* scrollingslider = new QSlider(200, 2000, 100, settings->scrollTimeout(), Qt::Horizontal, interactivitygroup);
	connect(scrollingspin, SIGNAL(valueChanged(int)), scrollingslider, SLOT(setValue(int)));
	connect(scrollingslider, SIGNAL(valueChanged(int)), scrollingspin, SLOT(setValue(int)));
	scrollinglayout->addWidget(scrollingslider);
	QHBoxLayout* scrollingenablelayout = new QHBoxLayout(scrollinglayout);
	scrollingenablelayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
	QCheckBox* scrollingcheck = new QCheckBox(tr("Enabled"), interactivitygroup);
	connect(scrollingcheck, SIGNAL(toggled(bool)), scrollingspin, SLOT(setEnabled(bool)));
	connect(scrollingcheck, SIGNAL(toggled(bool)), scrollingslider, SLOT(setEnabled(bool)));
	connect(scrollingcheck, SIGNAL(toggled(bool)), settings, SLOT(enableScrolling(bool)));
	connect(settings, SIGNAL(scrollingChange(bool)), scrollingcheck, SLOT(setChecked(bool)));
	scrollingenablelayout->addWidget(scrollingcheck);
	scrollinglayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Minimum));
	
	QHBoxLayout* browserlayout = new QHBoxLayout(interactivitylayout);
	QLabel* browserlabel = new QLabel(tr("Web Browser"), interactivitygroup);
	browserlabel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	browserlayout->addWidget(browserlabel);
	QComboBox* browsercombo = new QComboBox(interactivitygroup);
	browsercombo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
	browsercombo->insertStringList(BROWSERS());
	browsercombo->setEditable(true);
	connect(browsercombo, SIGNAL(textChanged(const QString&)), settings, SLOT(setBrowser(const QString&)));
	connect(settings, SIGNAL(browserChange(const QString&)), browsercombo, SLOT(setEditText(const QString&)));
	browserlayout->addWidget(browsercombo);
	QPushButton* browserbutton = new QPushButton(tr("Browse"), interactivitygroup);
	browserbutton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(browserbutton, SIGNAL(clicked()), this, SLOT(getBrowser()));
	browserlayout->addWidget(browserbutton);
	
	appearancelayout->addWidget(interactivitygroup);
	
	appearancelayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));
	
	this->addTab(appearancetab, tr("Appearance"));
}

void
ConfigDialog::getBrowser()
{
	QString browser = QFileDialog::getOpenFileName("", "", this, "", "Select Browser");
	if (!browser.isNull())
		settings->setBrowser(browser);
}

void
ConfigDialog::setSkin(const QString& skin)
{
	skincombo->setCurrentText(skin);
}

void
ConfigDialog::updateRssList(SiteList sites)
{
	rssupbutton->setEnabled(sites.count() > 1);
	rssdownbutton->setEnabled(sites.count() > 1);
	rssremovebutton->setEnabled(sites.count() != 0);
	
	QStringList order;
	
	for (unsigned int i = 0; i < sites.count(); i++) {
		Site site = sites[i];
		order.append(site.url);
		if (rssitems[site.url] == 0) {
			CheckBoxItem* item = new CheckBoxItem(settings, i, rsslist);
			item->setDragEnabled(true);
			item->setDropEnabled(true);
			rssitems.insert(site.url, item);
		} else {
			CheckBoxItem* item = rssitems[site.url];
			item->setText(0, site.title);
			item->setOn(site.enabled);
			item->setIndex(i);
		}
	}
	
	QDictIterator<CheckBoxItem> di(rssitems);
	while (di.current()) {
		if (!order.contains(di.currentKey())) {
			delete di.current();
			rssitems.remove(di.currentKey());
		}
		++di;
	}
	
	rsslist->sort();
}

void
ConfigDialog::addRss()
{
	bool ok;
	QString url = QInputDialog::getText("RSS", tr("What is the URL?"), QLineEdit::Normal, "http://", &ok, this);
	if (ok)
		adder->add(url);
}

void
ConfigDialog::removeRss()
{
	unsigned int index = ((CheckBoxItem*)(rsslist->currentItem()))->index();
	Site site = settings->sites()[index];
	settings->removeSite(site);
}

void
ConfigDialog::promoteRss()
{
	((CheckBoxItem*)(rsslist->currentItem()))->promote();
}

void
ConfigDialog::demoteRss()
{
	((CheckBoxItem*)(rsslist->currentItem()))->demote();
}

void
ConfigDialog::urlFailed(QString url)
{
	QMessageBox::warning(this, tr("Add"), url + tr(" is not a valid RSS URL!"));
}

void
ConfigDialog::faceChange(const QString& face)
{
	facecombo->setCurrentText(face);
}

void
ConfigDialog::sizeChange(int size)
{
	sizecombo->setCurrentText(QString::number(size));
}

void
ConfigDialog::changeSize(const QString& size)
{
	settings->setFontSize(size.toInt());
}

void
ConfigDialog::fgChange(QColor fg)
{
	QPixmap pm(100, facecombo->height() * 2 / 3);
	pm.fill(fg);
	fgbutton->setPixmap(pm);
}

void
ConfigDialog::changeFg()
{
	QColor newcolor = QColorDialog::getColor(settings->fg(), this);
	if (!newcolor.isValid())
		return;
	settings->setFgColor(newcolor);
}

void
ConfigDialog::bgChange(QColor bg)
{
	QPixmap pm(100, facecombo->height() * 2 / 3);
	pm.fill(bg);
	bgbutton->setPixmap(pm);
}

void
ConfigDialog::changeBg()
{
	QColor newcolor = QColorDialog::getColor(settings->bg(), this);
	if (!newcolor.isValid())
		return;
	settings->setBgColor(newcolor);
}

void
ConfigDialog::resizeEvent(QResizeEvent* e)
{
	QTabDialog::resizeEvent(e);
	fgChange(settings->fg());
	bgChange(settings->bg());
}

void
ConfigDialog::rssEnable(QListViewItem* i)
{
	if (i == 0)
		return;
	CheckBoxItem* item = (CheckBoxItem*)i;
	Site site = settings->sites()[item->index()];
	selsite = site;
	if (site.enabled == item->isOn())
		return;
	site.enabled = item->isOn();
	settings->appendSite(site);
}

QDragObject*
SiteListView::dragObject()
{
	CheckBoxItem* item = (CheckBoxItem*)selectedItem();
	QDragObject* d = new QTextDrag(item->site().url, this);
	return d;
}
