/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include <qt.h>
#include <qprocess.h>
#ifdef Q_WS_MACX
#include <Carbon/Carbon.h>
#endif

#include "mainwindow.h"
#include "paths.h"

MainWindow::MainWindow(QWidget* parent, const char* name)
#ifdef Q_WS_MACX
: QWidget(parent, name, Qt::WStyle_NoBorder|Qt::WMacNoSheet)
#else
: QWidget(parent, name, Qt::WStyle_NoBorder|Qt::WStyle_StaysOnTop|Qt::WX11BypassWM)
#endif
{
	QPixmap icon(SHARED+"rt.xpm");
	this->setIcon(icon);
	
	this->helper = new MinimizeHelper();
	connect(helper, SIGNAL(aboutToShow()), this, SLOT(show()));
	
	cache = new Cache(this);
	connect(cache, SIGNAL(cacheUpdating()), this, SLOT(beginReload()));
	connect(cache, SIGNAL(cacheCurrent()), this, SLOT(endReload()));
	
	settings = new Settings(this);
	
	connect(settings, SIGNAL(skinChange(const QString&)),
	        this, SLOT(setSkin(const QString&)));
	connect(settings, SIGNAL(headlineTimeoutChange(int)),
	        this, SLOT(setTimeout(int)));
	connect(settings, SIGNAL(sitesChange(SiteList)),
	        this, SLOT(newSites(SiteList)));
	connect(settings, SIGNAL(cacheTimeoutChange(int)),
	        cache, SLOT(setTimeout(int)));
	connect(cache, SIGNAL(urlInvalid(QString)),
	        settings, SLOT(disableSite(QString)));
	connect(cache, SIGNAL(rssInvalid(QString)),
	        settings, SLOT(disableSite(QString)));
	connect(cache, SIGNAL(rssTitle(QString, QString)),
	        settings, SLOT(setTitle(QString, QString)));
	connect(cache, SIGNAL(urlModified(QString, QString)),
	        settings, SLOT(modifyUrl(QString, QString)));
	
	tabulator = new Tabulator(settings, cache, this);
	connect(tabulator, SIGNAL(fileReady(QString)), this, SLOT(openUrl(QString)));
	connect(this, SIGNAL(urlClosed(QString)), tabulator, SLOT(clean(QString)));
	
	confdialog = new ConfigDialog(settings, cache, this);
	
	manager = new Manager(cache, this);
	connect(manager, SIGNAL(itemAvailable(Item)),
	        this, SLOT(display(Item)));
	connect(manager, SIGNAL(itemLoading()),
	        this, SLOT(wait()));
	connect(manager, SIGNAL(noItems()),
	        this, SLOT(error()));
	
	connect(settings, SIGNAL(sitesChange(SiteList)),
	        manager, SLOT(setSites(SiteList)));
	
	ticker = new Ticker(this);
	connect(ticker, SIGNAL(urlRequest(QString)), this, SLOT(openUrl(QString)));
	connect(settings, SIGNAL(bgChange(QColor)),
	        ticker, SLOT(setBgColor(QColor)));
	connect(settings, SIGNAL(fgChange(QColor)),
	        ticker, SLOT(setFgColor(QColor)));
	connect(settings, SIGNAL(fontChange(QFont)),
	        ticker, SLOT(setFont(QFont)));
	connect(settings, SIGNAL(scrollTimeoutChange(int)),
	        ticker, SLOT(setTimeout(int)));
	connect(settings, SIGNAL(scrollingChange(bool)),
	        ticker, SLOT(enableScrolling(bool)));
	connect(settings, SIGNAL(tooltipsChange(bool)),
	        ticker, SLOT(enableTooltips(bool)));
	connect(settings, SIGNAL(underliningChange(bool)),
	        ticker, SLOT(enableUnderlining(bool)));
	
	jumpmenu = new QPopupMenu(this);
	connect(jumpmenu, SIGNAL(aboutToShow()), this, SLOT(fillJumpMenu()));
	connect(jumpmenu, SIGNAL(activated(int)), this, SLOT(jumpToIndex(int)));
	
	popup = new QPopupMenu(this);
	popup->insertItem(tr("Reload latest information"), this, SLOT(reload()));
	popup->insertItem(tr("Tabulate and view"), tabulator, SLOT(create()));
	popup->insertSeparator();
	popup->insertItem(tr("OPML Import"), this, SLOT(opmlimport()));
	popup->insertItem(tr("OPML Export"), this, SLOT(opmlexport()));
	popup->insertItem(tr("Properties"), confdialog, SLOT(show()));
	popup->insertSeparator();
	popup->insertItem(tr("Version Information"), this, SLOT(verInfo()));
	popup->insertItem(tr("Quit"), this, SLOT(quit()));
	popup->insertItem(tr("Jump"), jumpmenu);
	
	adder = new Adder(cache, false, this);
	clipboard = QApplication::clipboard();
	connect(clipboard, SIGNAL(dataChanged()), this, SLOT(clipboardChanged()));
	connect(adder, SIGNAL(validSite(Site)), settings, SLOT(appendSite(Site)));
	
	connect(&timer, SIGNAL(timeout()), this, SLOT(headlineTimeout()));
	
	reloading = false;
	pausing = false;
	waiting = false;
	moving = false;
	left = true;
	skin = 0;
	
	this->settings->load();
	move(settings->startPos());
	manager->next();
}

void
MainWindow::setSkin(const QString& skin)
{
	if (this->skin != 0)
		delete this->skin;
	
	this->skin = new Skin(skin);
	this->setMinimumSize(this->skin->base().size());
	this->setMaximumSize(this->skin->base().size());
	this->setBackgroundPixmap(this->skin->base());
	this->setMask(this->skin->mask());
	this->ticker->setGeometry(this->skin->ticker());
}

void
MainWindow::verInfo()
{
	QMessageBox::about(this, tr("Version Information"), tr("RabbitTicker 1.0 Beta 3"));
}

void
MainWindow::openUrl(QString url)
{
	cleanProcesses();
	QProcess* x = new QProcess(this);
	QStringList args;
	int index = 0;
	QString browser = settings->browser();
	while (browser.find(" ", index) != -1) {
		args.append(browser.mid(index, browser.find(" ", index) - index));
		index = browser.find(" ", index) + 1;
	}
	args.append(browser.mid(index));
	args.append(url);
	bool success;
#ifdef Q_WS_MACX
	if (browser == "open") {
		x->setArguments(args);
		success = x->start();
	} else {
		FSRef appRef;
		FSRef itemRef;
		LSLaunchFSRefSpec launchFSRefSpec;
		OSStatus err = FSPathMakeRef((const char*)browser, &appRef, NULL);
		success =  (err == noErr);
		qDebug(QString::number(QDateTime::currentDateTime().toTime_t())+"  MainWindow::openUrl FSPathMakeRef(browser:" + (const char*)browser);
		if (success) {
			err = FSPathMakeRef((const char*)url, &itemRef, NULL);
			success =  (err == noErr);
			qDebug(QString::number(QDateTime::currentDateTime().toTime_t())+" MainWindow::openUrl FSPathMakeRef(url):" + (const char*)url);
		}
		if (success) {
			launchFSRefSpec.appRef = &appRef;
			launchFSRefSpec.numDocs = 1;
			launchFSRefSpec.itemRefs = &itemRef;
			launchFSRefSpec.passThruParams = nil;
			launchFSRefSpec.launchFlags = 0;
			launchFSRefSpec.asyncRefCon = nil;
			err = LSOpenFromRefSpec(&launchFSRefSpec, NULL);
			success =  (err == noErr);
			qDebug(QString::number(QDateTime::currentDateTime().toTime_t())+" MainWindow::openUrl LSOpenFromRefSpec()");
		}
	}
#else
	x->setArguments(args);
	success = x->start();
#endif
	moving = false;
	if (success) {
		if (processes[url] != 0) {
			delete processes[url];
			processes.remove(url);
		}
		processes.insert(url, x);
	} else {
		QMessageBox::critical(this, "", tr("Could not execute \"") + settings->browser() + tr("\".\nPlease check your configuration."));
		delete x;
	}
}

void
MainWindow::cleanProcesses(bool force)
{
	QDictIterator<QProcess> i(processes);
	while (i.current()) {
		QProcess* p = i.current();
		if (!p->isRunning() || force) {
			emit urlClosed(i.currentKey());
			processes.remove(i.currentKey());
			delete p;
		} else {
			++i;
		}
	}
}

void
MainWindow::headlineTimeout()
{
	left = true;
	if (wheeling) {
		wheeling = false;
		ticker->setTimeout(settings->scrollTimeout());
	}
	manager->next();
}

void
MainWindow::display(Item item)
{
	if (waiting) {
		waiting = false;
		ticker->reset();
	}
	if (wheeling)
		ticker->setTimeout(200);
	
	if (left)
		ticker->scrollLeft(item);
	else
		ticker->scrollRight(item);
	timer.start(settings->headlineTimeout() * 1000);
}

void
MainWindow::wait()
{
	waiting = true;
	Item item;
	item.channel.title = tr("Please wait...");
	ticker->reset();
	ticker->scrollLeft(item);
}

void
MainWindow::error()
{
	waiting = true;
	Item item;
	item.channel.title = tr("No sites available!");
	ticker->reset();
	ticker->scrollLeft(item);
}

void
MainWindow::setTimeout(int timeout)
{
	this->timeout = timeout * 1000;
	timer.start(this->timeout);
}

void
MainWindow::reload()
{
	cache->invalidate();
	manager->next();
}

void
MainWindow::beginReload()
{
	reloading = true;
	repaint(skin->reload());
}

void
MainWindow::endReload()
{
	reloading = false;
	repaint(skin->reload());
}

void
MainWindow::newSites(SiteList)
{
	if (waiting) {
		timer.start(50);
	}
}

void
MainWindow::clipboardChanged()
{
	QString url = clipboard->text(QClipboard::Clipboard);
	adder->add(url);
}

void
MainWindow::mousePressEvent(QMouseEvent* e)
{
	moving = false;
	moved = false;
	
	if (e->button() == Qt::LeftButton) {
		if (skin->close().contains(e->pos())) {
			quit();
		
		} else if (skin->minimize().contains(e->pos())) {
			hide();
			helper->showMinimized();
			timer.stop();
		
		} else if (skin->reload().contains(e->pos()) && !reloading) {
			reload();
		
		} else if (skin->pause().contains(e->pos())) {
			pausing = !pausing;
			if (pausing)
				timer.stop();
			else
				timer.start(settings->headlineTimeout() * 1000);
			repaint();
		} else if (skin->tabulate().contains(e->pos())) {
			tabulator->create();
		
		} else {
			mousepos = e->pos();
			moving = true;
		}
	} else {
		e->ignore();
	}
}

void
MainWindow::wheelEvent(QWheelEvent *e)
{
	left = false;
	int times = e->delta() / 120;
	if (times < 0) {
		left = true;
		times = -times;
	}
	
	for (int i = 0; i < times; i++)
		scroll();
}

void
MainWindow::keyPressEvent(QKeyEvent* e)
{
	if (e->text() == "+" || e->key() == Qt::Key_Left) {
		e->accept();
		left = true;
		scroll();
	} else if (e->text() == "-" || e->key() == Qt::Key_Right) {
		e->accept();
		left = false;
		scroll();
	} else {
		e->ignore();
	}
}

void
MainWindow::scroll()
{
	if (waiting)
		return;
	
	timer.stop();
	wheeling = true;
	
	if (left) {
		manager->next();
	} else {
		manager->previous();
	}
}

void
MainWindow::contextMenuEvent(QContextMenuEvent* e)
{
	moving = false;
	popup->setItemEnabled(popup->idAt(0), !reloading);
	popup->exec(e->globalPos());
}

void
MainWindow::paintEvent(QPaintEvent* e)
{
	QPainter p(this);
	
	if (e->rect().intersects(skin->reload()))
		p.drawPixmap(skin->reload(), (reloading ? skin->reloadon() : skin->reloadoff()));
	
	if (e->rect().intersects(skin->pause()))
		p.drawPixmap(skin->pause(), (pausing ? skin->pauseon() : skin->pauseoff()));
}

void
MainWindow::mouseMoveEvent(QMouseEvent* e)
{
	if (moving) {
		move(e->globalPos() - mousepos);
		moved = true;
	}
}

void
MainWindow::mouseReleaseEvent(QMouseEvent* e)
{
	moving = false;
	if (moved)
		moved = false;
	else
		setActiveWindow();
}

void
MainWindow::showEvent(QShowEvent*)
{
	helper->hide();
	if (!pausing)
		timer.start(settings->headlineTimeout() * 1000);
}

void
MainWindow::quit()
{
	cleanProcesses(true);
	settings->setStartPos(pos());
	settings->save();
	qApp->quit();
}

void
MainWindow::opmlimport()
{
  QString s = QFileDialog::getOpenFileName(QString::null, tr("OPML Files (*.xml *.opml)"),
	                                         this, 0, tr("OPML Import" ));
	if (s != QString::null)
		settings->opmlimport(s);
}

void
MainWindow::opmlexport()
{
  QString s = QFileDialog::getSaveFileName(QString::null, tr("OPML Files (*.xml *.opml)"),
	                                         this, 0, tr("OPML Export" ));
	if (s != QString::null)
		settings->opmlexport(s);
}

void
MainWindow::fillJumpMenu()
{
	jumpmenu->clear();
	for (unsigned int i = 0; i < settings->sites().count(); i++) {
		if (settings->sites()[i].enabled)
			jumpmenu->insertItem(settings->sites()[i].title, i);
	}
}

void
MainWindow::jumpToIndex(int index)
{
	wheeling = true;
	manager->jump(settings->sites()[index].url);
	manager->next();
}
