/***************************************************************************
 *   Copyright (C) 2004 by Work@                                           *
 *   jstubbs@work-at.co.jp                                                 *
 *                                                                         *
 *   This program may be distributed under the terms of the Q Public       *
 *   License as defined by Trolltech AS of Norway and appearing in the     *
 *   file LICENSE.QPL included in the packaging of this file.              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY ; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *
 ***************************************************************************/

#include <qimage.h>
#include <qsettings.h>
#include <qdom.h>

#include "settings.h"
#include "paths.h"

Settings::Settings(QObject* parent, const char* name)
	: QObject(parent, name)
{
	QDir dir(USER);
	if (!dir.exists())
		dir.mkdir(USER);
	_autocolors = true;
	_bg = QColor(255, 255, 255);
	_fg = QColor(0, 0, 0);
	_autofont = true;
	_fontface = QFont::defaultFont().family();
	_fontsize = QFont::defaultFont().pointSize();
	_skin = "Default";
	_cachetimeout = 60;
	_headlinetimeout = 15;
	_scrolltimeout = 1000;
	_monitoring = true;
	_underlining = true;
	_tooltips = true;
	_scrolling = true;
	_sites.clear();
}

void
Settings::setBrowser(const QString& browser)
{
	if (_browser != browser)
		emit browserChange(browser);
	_browser = browser;
}

QString
Settings::browser() const
{
	return _browser;
}

bool
Settings::autoColors() const
{
	return _autocolors;
}

QColor
Settings::bg() const
{
	return _bg;
}

QColor
Settings::fg() const
{
	return _fg;
}

bool
Settings::autoFont() const
{
	return _autofont;
}

QString
Settings::fontFace() const
{
	return _fontface;
}

int
Settings::fontSize() const
{
	return _fontsize;
}

QString
Settings::skin() const
{
	return _skin;
}

int
Settings::cacheTimeout() const
{
	return _cachetimeout;
}

int
Settings::headlineTimeout() const
{
	return _headlinetimeout;
}

int
Settings::scrollTimeout() const
{
	return _scrolltimeout;
}

SiteList
Settings::sites() const
{
	return _sites;
}

bool
Settings::monitoring() const
{
	return _monitoring;
}

bool
Settings::underlining() const
{
	return _underlining;
}

bool
Settings::tooltips() const
{
	return _tooltips;
}

bool
Settings::scrolling() const
{
	return _scrolling;
}

void
Settings::modifyUrl(QString prev, QString curr)
{
	for (uint i = 0; i < _sites.count(); i++) {
		Site site = _sites[i];
		if (site.url == prev) {
			site.url = curr;
			_sites[i] = site;
			emit sitesChange(_sites);
			return;
		}
	}
}

void
Settings::opmlimport(QString path)
{
	QFile f(path);
	f.open(IO_ReadOnly);
	QDomDocument doc;
	doc.setContent(&f);
	QDomNodeList nodes = doc.elementsByTagName("outline");
	if (nodes.count() == 0) {
		return;
	}
	
	for (unsigned int idx = 0; idx < nodes.count(); idx++) {
		QDomElement node = nodes.item(idx).toElement();
		Site site;
		site.url = node.attribute("xmlUrl");
		site.title = node.attribute("title");
		site.enabled = (node.attribute("load") == "yes");
		if (!_sites.contains(site))
			_sites.append(site);
	}
	f.close();
	
	emit sitesChange(_sites);
}

void
Settings::opmlexport(QString path)
{
	QDomDocument doc;
	QDomElement opmlnode = doc.createElement("opml");
	opmlnode.setAttribute("version", "1.0");
	doc.appendChild(opmlnode);
	QDomElement headnode = doc.createElement("head");
	opmlnode.appendChild(headnode);
	QDomElement titlenode = doc.createElement("title");
	titlenode.appendChild(doc.createTextNode("Rabbit Ticker List"));
	QDomElement namenode = doc.createElement("ownerName");
	namenode.appendChild(doc.createTextNode("Work@Inc. with Yomiusa.com"));
	headnode.appendChild(titlenode);
	QDomElement emailnode = doc.createElement("ownerEmail");
	emailnode.appendChild(doc.createTextNode("jstubbs@work-at.co.jp"));
	emailnode.appendChild(emailnode);
	QDomElement bodynode = doc.createElement("body");
	opmlnode.appendChild(bodynode);
	for (unsigned int i = 0; i < _sites.count(); i++) {
		Site site = _sites[i];
		QDomElement outlinenode = doc.createElement("outline");
		outlinenode.setAttribute("text", site.title);
		outlinenode.setAttribute("title", site.title);
		outlinenode.setAttribute("type", "rss");
		outlinenode.setAttribute("xmlUrl", site.url);
		if (site.enabled)
			outlinenode.setAttribute("load", "yes");
		else
			outlinenode.setAttribute("load", "no");
		bodynode.appendChild(outlinenode);
	}
	QFile f(path);
	f.open(IO_WriteOnly | IO_Truncate);
	QTextStream ts(&f);
	ts.setEncoding(QTextStream::UnicodeUTF8);
	ts << "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
	ts << doc.toString() << "\n";
	f.close();
}

void
Settings::load()
{
	
	_sites.clear();
	opmlimport(USER+"config.xml");
	
	ConfigFile settings(USER+"rabbittickerrc");
	settings.read();
	
	_cachetimeout = settings.get("CacheTimeout", 3600);
	_monitoring = settings.get("ClipboardMonitoring", false);
	
	_skin = settings.get("Skin", QString("Default"));
	QFont font = QFont::defaultFont();
	QString fontdesc = settings.get("Font", QString(""));
	if (fontdesc != "")
		font.fromString(fontdesc);
	_fontface = font.family();
	_fontsize = font.pointSize();
	_autofont = settings.get("DefaultFont", true);
	_fg = settings.get("TextColor", 0);
	_bg = settings.get("BackgroundColor", -1);
	_autocolors = settings.get("DefaultColors", true);
	_headlinetimeout = settings.get("TickInterval", 15);
	_underlining = settings.get("UnderlineLinks", true);
	_tooltips = settings.get("ShowTooltips", true);
	_scrolltimeout = settings.get("ScrollSpeed", 1000);
	_scrolling = settings.get("ScrollText", true);
	_browser = settings.get("Browser", QString(""));
	_startpos = QPoint(settings.get("WinX", (int)0), settings.get("WinY", (int)0));
	if (_sites.count() == 0)
		opmlimport(SHARED+"default.opml");
	if (_browser == "")
		_browser = BROWSERS()[0];
	emit skinChange(_skin);
	emit cacheTimeoutChange(_cachetimeout);
	emit headlineTimeoutChange(_headlinetimeout);
	emit scrollTimeoutChange(_scrolltimeout);
	emit sitesChange(_sites);
	emit monitoringChange(_monitoring);
	emit underliningChange(_underlining);
	emit tooltipsChange(_tooltips);
	emit scrollingChange(_scrolling);
	emit autoFontChange(_autofont);
	emit fontFaceChange(_fontface);
	emit fontSizeChange(_fontsize);
	emit autoColorsChange(_autocolors);
	emit sitesChange(_sites);
	emit browserChange(_browser);
	emitFontChange();
	emitColorChange();
}

void
Settings::save()
{
	opmlexport(USER+"config.xml");
	
	ConfigFile settings(USER+"rabbittickerrc");
	settings.set("CacheTimeout", (int)_cachetimeout);
	settings.set("ClipboardMonitoring", _monitoring);
	settings.set("Skin", _skin);
	QFont font = QFont::defaultFont();
	font.setFamily(_fontface);
	font.setPointSize(_fontsize);
	settings.set("Font", font.toString());
	settings.set("DefaultFont", _autofont);
	settings.set("TextColor", (int)_fg.rgb());
	settings.set("BackgroundColor", (int)_bg.rgb());
	settings.set("DefaultColors", _autocolors);
	settings.set("TickInterval", (int)_headlinetimeout);
	settings.set("UnderlineLinks", _underlining);
	settings.set("ShowTooltips", _tooltips);
	settings.set("ScrollSpeed", (int)_scrolltimeout);
	settings.set("ScrollText", _scrolling);
	settings.set("Browser", _browser);
	settings.set("WinX", _startpos.x());
	settings.set("WinY", _startpos.y());
	settings.write();
}

void
Settings::setAutoColors(bool enabled)
{
	if (_autocolors == enabled)
		return;
	_autocolors = enabled;
	emit autoColorsChange(enabled);
	emitColorChange();
}

void
Settings::emitColorChange()
{
	if (_autocolors) {
		Skin skin(_skin);
		QImage image = skin.base().convertToImage();
		long r = 0;
		long g = 0;
		long b = 0;
		for (int x = skin.ticker().left(); x < skin.ticker().right(); x++) {
			for (int y = skin.ticker().top(); y < skin.ticker().bottom(); y++) {
				QColor color(image.pixel(x, y));
				r += color.red();
				g += color.green();
				b += color.blue();
			}
		}
		r /= skin.ticker().width() * skin.ticker().height();
		g /= skin.ticker().width() * skin.ticker().height();
		b /= skin.ticker().width() * skin.ticker().height();
		QColor bg(r, g, b);
		emit bgChange(bg);
		int h;
		int s;
		int v;
		bg.getHsv(&h, &s, &v);
		if (v < 64)
			v = 255;
		else if (v > 192)
			v = 0;
		else {
			v += 127;
			if (v > 255)
				v -= 256;
		}
		h += 180;
		if (h > 359)
			h -= 360;
		QColor fg;
		fg.setHsv(h, s, v);
		emit fgChange(fg);
	} else {
		emit bgChange(_bg);
		emit fgChange(_fg);
	}
}

void
Settings::setBgColor(QColor bg)
{
	if (bg == _bg)
		return;
	_bg = bg;
	
	if (_autocolors)
		return;
	
	emit bgChange(bg);
}

void
Settings::setFgColor(QColor fg)
{
	if (fg == _fg)
		return;
	_fg = fg;
	
	if (_autocolors)
		return;
	
	emit fgChange(fg);
}

void
Settings::setAutoFont(bool enabled)
{
	if (enabled == _autofont)
		return;
	_autofont = enabled;
	emit autoFontChange(enabled);
	emitFontChange();
}

void
Settings::emitFontChange()
{
	if (_autofont)
		emit fontChange(QFont::defaultFont());
	else {
		QFont font;
		font.setFamily(_fontface);
		font.setPointSize(_fontsize);
		emit fontChange(font);
	}
}

void
Settings::setFontFace(const QString& face)
{
	if (face == _fontface)
		return;
	_fontface = face;
	
	emit fontFaceChange(face);
	
	if (_autofont)
		return;
	
	emitFontChange();
}

void
Settings::setFontSize(int size)
{
	if (size == _fontsize)
		return;
	_fontsize = size;
	
	emit fontSizeChange(size);
	
	if (_autofont)
		return;
	
	emitFontChange();
}

void
Settings::setSkin(const QString& skin)
{
	if (skin == _skin)
		return;
	_skin = skin;
	emit skinChange(skin);
	if (_autocolors)
		emitColorChange();
}

void
Settings::setCacheTimeout(int timeout)
{
	if (timeout == _cachetimeout)
		return;
	_cachetimeout = timeout;
	emit cacheTimeoutChange(timeout);
}

void
Settings::setHeadlineTimeout(int timeout)
{
	if (timeout == _headlinetimeout)
		return;
	_headlinetimeout = timeout;
	emit headlineTimeoutChange(timeout);
}

void
Settings::setScrollTimeout(int timeout)
{
	if (timeout == _scrolltimeout)
		return;
	_scrolltimeout = timeout;
	emit scrollTimeoutChange(timeout);
}

void
Settings::appendSite(Site site)
{
	if (_sites.contains(site)) {
		int index = _sites.findIndex(site);
		_sites[index] = site;
	} else {
		_sites.append(site);
	}
	emit sitesChange(_sites);
}

void
Settings::indexSite(Site site, int index)
{
	while (_sites.contains(site))
		_sites.remove(site);
	_sites.insert(_sites.at(index), site);
	emit sitesChange(_sites);
}

void
Settings::removeSite(Site site)
{
	while (_sites.contains(site))
		_sites.remove(site);
	emit sitesChange(_sites);
}

void
Settings::enableMonitoring(bool enabled)
{
	if (_monitoring == enabled)
		return;
	_monitoring = enabled;
	emit monitoringChange(enabled);
}

void
Settings::enableUnderlining(bool enabled)
{
	if (_underlining == enabled)
		return;
	_underlining = enabled;
	emit underliningChange(enabled);
}

void
Settings::enableTooltips(bool enabled)
{
	if (_tooltips == enabled)
		return;
	_tooltips = enabled;
	emit tooltipsChange(enabled);
}

void
Settings::enableScrolling(bool enabled)
{
	if (_scrolling == enabled)
		return;
	_scrolling = enabled;
	emit scrollingChange(enabled);
}

void
Settings::disableSite(QString url)
{
	for (unsigned int i = 0; i < _sites.count(); i++) {
		if (_sites[i].url == url) {
			Site site = _sites[i];
			if (!site.enabled)
				return;
			site.enabled = false;
			if (site.title.left(8) != tr("*ERROR* "))
				site.title = tr("*ERROR* ") + site.title;
			_sites[i] = site;
			emit sitesChange(_sites);
			return;
		}
	}
}

void
Settings::setTitle(QString url, QString title)
{
	for (unsigned int i = 0; i < _sites.count(); i++) {
		if (_sites[i].url == url) {
			Site site = _sites[i];
			if (site.title == title)
				return;
			site.title = title;
			_sites[i] = site;
			emit sitesChange(_sites);
			return;
		}
	}
}

QPoint
Settings::startPos() const
{return _startpos;}

void
Settings::setStartPos(const QPoint& pos)
{_startpos = pos;}
